'' ============================================================
'' 3D Solid Body Simulation Sampler
''
'' Copyright (C) 2006 - Jeff Marshall (coder@execulink.com)
''
'' License: 1) Program is provided as-is.  
''          2) There is no warranty or guarantee.
''          3) There are no other conditions.
''
'' Feel free to use, re-use, or abuse :)
''
'' ============================================================
''
'' Requires FreeBasic Compiler
''
'' Compile using:
''   fbc atoms.bas atomseng.bas vector.bas
''
'' ============================================================

Option Explicit
const False = 0, True = Not False

#include "GL/gl.bi"
#include "GL/glu.bi"
#include "atomseng3.bi"

'These variables are set by default, but may be changed here
'ATOMSIZE = 4
'ATOMSPACE = 10
'MAXSPEED = 5
'ZGRAVITY = -.004
'YGRAVITY = 0
'XGRAVITY = 0
'BOUNDDECAY = .99
'DEFORMFACTOR = .4
'DEFORMTHRESHOLD = 1
'BREAKTHRESHOLD = 2
'STIFFNESS = 0.5
'BONDTHRESHOLD = 0.5  '' 0.2 was used in atom_stick.bas
'SHOWATOMS = 1
'SHOWBONDS = 1

dim as integer grabatom, mx, my, mw, mb, oldmb, i, j
dim as single smx, smy, viewdist = 300, viewangle = 0.75, viewheight = 50, lookheight = 25, aa
dim as string k, skipframe

screen 18, 32, , 2

glViewport 0, 0, 640, 480
glMatrixMode GL_PROJECTION
glLoadIdentity
gluPerspective 45.0, 640.0/480.0, 0.1, 10000.0
glMatrixMode GL_MODELVIEW
glLoadIdentity

glShadeModel GL_SMOOTH
glClearColor 0.0, 0.0, 0.0, 1.0
glClearDepth 1.0
glEnable GL_DEPTH_TEST
glDepthFunc GL_LEQUAL
glHint GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST

ReStart:

Randomize Timer
ClearAllAtoms

If (command(1)) > "" then
	i = 1
	while command(i) > ""
		LoadSettings command(i)	
		i += 1
	wend
else

	'' Various shapes
	CreateBlock  0, 0,  100,  10, 2, 4, rgb(255,0,0), 0.1,0.1,0.1
	CreateBlock  0, 0,  1000,  4, 8, 2, rgb(255,255,0), 0.5,0.1,0.5
	CreateBlock  0, 0,  5000,  4, 8, 2, rgb(0,255,255), 0.5,0.5,0.5

end if

grabatom = 0

do
	oldmb = mb
	GetMouse mx, my, mw, mb
	smx = mx - 320
	smy = 480 - my

	if mb <> 0 and oldmb = 0 then
		grabatom = FindAtom(smx, 0, smy)
	elseif mb = 0 then
		grabatom = 0
	end if

	if grabatom > 0 then
		if a(grabatom).ex = TRUE then
			a(grabatom).v.x = smx - a(grabatom).p.x
			a(grabatom).v.y = smy - a(grabatom).p.y
			if (mb and 2) then
				for i = 1 to NBONDS
					if b(i).a1 = grabatom then
						a(b(i).a2).v = a(grabatom).v
					elseif b(i).a2 = grabatom then
						a(b(i).a1).v = a(grabatom).v
					end if
				next i
			end if
		else
			grabatom = 0
		end if
	end if

	UpdateAtoms  

	glClear GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT
	glLoadIdentity
	gluLookAt viewdist * cos(viewangle), viewdist * sin(viewangle),viewheight,0,0,lookheight,0,0,1

	glPushMatrix
	for i = -320 to 320 step 20
		glColor3f 0.5,0,0
		glBegin( GL_LINES )
			glVertex3f( i, 320, 0 )
			glVertex3f( i, -320, 0 )
			glVertex3f( 320, i, 0 )
			glVertex3f( -320, i, 0 )
		glEnd()
	next i
	glPopMatrix

	'DrawAtoms
	if (SHOWATOMS) then
		' Draw all the atoms
		for i = 1 TO NATOMS
			if a(i).ex = TRUE then
				aa = ATOMSIZE / 2
				glPushMatrix
				glTranslatef a(i).p.x, a(i).p.y, a(i).p.z
				glColor3f a(i).clr.x / 2, a(i).clr.y / 2, a(i).clr.z / 2
				glBegin GL_QUADS
					glVertex3f -aa, aa, 0
					glVertex3f  aa, aa, 0
					glVertex3f  aa,-aa, 0
					glVertex3f -aa,-aa, 0

					glVertex3f -aa, 0,-aa
					glVertex3f  aa, 0, aa
					glVertex3f  aa, 0,-aa
					glVertex3f -aa, 0,-aa

					glVertex3f  0, -aa, aa
					glVertex3f  0,  aa, aa
					glVertex3f  0,  aa,-aa
					glVertex3f  0, -aa,-aa

				glEnd
				glPopMatrix
			end if
		next i
	end if

	glPushMatrix
	if (SHOWBONDS) then
		' Draw All the Bonds
		for i = 1 TO NBONDS
			if b(i).ex = TRUE then
				glColor3f b(i).clr.x, b(i).clr.y, b(i).clr.z
				glBegin( GL_LINES )
					glVertex3f( a(b(i).a1).p.x, a(b(i).a1).p.y, a(b(i).a1).p.z )
					glVertex3f( a(b(i).a2).p.x, a(b(i).a2).p.y, a(b(i).a2).p.z )
				glEnd()
			end if
		next i
	end if
	glPopMatrix

	flip
	
	k = inkey
	select case k
	case "R", "r"
		goto Restart
	case " "
		for i = 1 TO NATOMS
			if a(i).ex = TRUE then
				VSet a(i).v, (rnd - 0.5) * MAXSPEED * 2, (rnd - 0.5) * MAXSPEED * 2, (rnd - 0.5) * MAXSPEED * 2
			end if
		next i
	case "Q", "q"
		lookheight = lookheight + 10
	case "A", "a"
		lookheight = lookheight - 10
	case "W", "w"
		viewheight = viewheight + 10
	case "S", "s"
		viewheight = viewheight - 10
	case "X", "x"
		viewangle = viewangle + 0.1
	case "Z", "z"
		viewangle = viewangle - 0.1    
	case "+"
		viewdist = viewdist - 10
	case "-"
		viewdist = viewdist + 10
	end select

loop until k = chr(27)

screen 0
end
